import click
from docker import APIClient, errors
import dockerpty
from rich import print




@click.command("start")
@click.argument("name")
def start(name: str) -> None:
    """Resume working in an environment."""
    try:
        client = APIClient(base_url="unix://var/run/docker.sock")
        containers = client.containers(all=True, filters={"name": name})
        if containers:
            dockerpty.start(client, containers[0])
        else:
            print(f":ghost: No freshenv environment called [bold]{name}[/bold] found.")
    except errors.DockerException:
        print(":cross_mark_button: Docker not installed or running. ")
    except Exception as e:
        print(f"Unknown exception: {e}")



from typing import Dict, List
import click
from docker import APIClient, errors
from rich import print
client: APIClient = None



def count_environents() -> int:
    return len(get_list_environments())


def get_list_environments() -> List[Dict]:
    environment_list = []
    client = APIClient(base_url="unix://var/run/docker.sock")
    environment_list = client.containers(all=True,filters={"label": "maintainer=Raiyan Yahya <raiyanyahyadeveloper@gmail.com>"})
    return environment_list


@click.command("view")
def view() -> None:
    """View local freshenv managed environments."""
    try:
        container_list = get_list_environments()
        if not container_list:
            print(":computer: No freshenv environments found.")
        for container in container_list:
            if "Exited" in container.get("Status"):  # type: ignore
                img = ":arrow_down: "
            else:
                img = ":arrow_forward: "
            print(
                img,
                "Name: [bold blue]" + container.get("Names")[0] + "[/bold blue]",  # type: ignore
                "| Flavour: [bold blue]" + container.get("Image").split("/")[-1] + "[/bold blue]",  # type: ignore
                "| State: [bold blue]" + container.get("Status") + "[/bold blue]",  # type: ignore
            )
    except errors.DockerException:
        print(":cross_mark_button: Docker not installed or running. ")
    except Exception as e:
        print(f"Unknown exception: {e}")


import click
from docker import APIClient, errors
from rich import print




@click.command("clean")
@click.option('--force', '-f', is_flag=True, help="Force remove freshenv flavours and environments.")
def clean(force: bool) -> None:
    """Remove all freshenv flavours and environments."""
    try:
        client = APIClient(base_url="unix://var/run/docker.sock")
        freshenv_containers = client.containers(all=True, filters={"name": "freshenv_*"})
        for container in freshenv_containers:
            client.remove_container(container=container['Id'], force=force)
        images_list = client.images(filters={"label": "maintainer=Raiyan Yahya <raiyanyahyadeveloper@gmail.com>"})
        for image in images_list:
            client.remove_image(image=image['Id'], force=force)
        print(":boom: freshenv flavours and environments removed.")
    except errors.APIError as e:
        if e.status_code == 409:
            print(":runner: Found a [bold green]running[/bold green] environment. Close the session first or use the [bold blue]--force[/bold blue] flag.")
        else:
            raise Exception(e)
    except errors.DockerException:
        print(":cross_mark_button: Docker not installed or running. ")
    except Exception as e:
        print(f"Unknown exception: {e}")



import click
from freshenv import provision, view, start, remove, check, clean, flavours


@click.group()
@click.version_option()
def cli() -> None:
    """A cli to provision and manage local developer environments."""


cli.add_command(provision.provision)
cli.add_command(view.view)
cli.add_command(start.start)
cli.add_command(remove.remove)
cli.add_command(check.check)
cli.add_command(clean.clean)
cli.add_command(flavours.flavours)


from click import Option, BadParameter
from ast import literal_eval

class PythonLiteralOption(Option):

    def type_cast_value(self, ctx, value):
        try:
            return literal_eval(value)
        except BadParameter:
            raise BadParameter(value)


import click
from rich import pretty, print
from urllib.request import urlopen
from json import loads



@click.command("flavours")
def flavours() -> None:
    """Show all available flavours for provisioning."""
    gist_reponse = urlopen("https://api.github.com/gists/c4709c540a7c29616c771ab642ed2b8b")
    if gist_reponse.getcode() == 200:
        gist_data = loads(gist_reponse.read().decode("utf-8"))
        flavours = loads(gist_data["files"]["fr-flavours.json"]["content"])
        print(f":mag: Found {len(flavours['fr-flavours'])} flavours:")
        pretty.pprint(flavours["fr-flavours"])
    else:
        print(":heavy_exclamation_mark: Could not fetch flavours.")
        exit(1)